package net.davidc.egp.snaxe;

import java.io.IOException;
import java.util.Comparator;

/**
 * @author David Croft (david@davidc.net)
 * @version $Id: Player.java 26 2011-09-18 16:24:33Z david $
 */
public abstract class Player
{
  private String name;
  private long queueJoinTime;

  public abstract void sendMessage(String message) throws IOException;

  public String getName()
  {
    return name;
  }

  public void setName(String name)
  {
    if (name == null || name.length() == 0) {
      name = "Player" + System.identityHashCode(this);
    }

    if (name.length() > 17) {
      name = name.substring(0, 14) + "...";
    }

    this.name = name;
  }

  public long getQueueJoinTime()
  {
    return queueJoinTime;
  }

  public void setQueueJoinTime(long queueJoinTime)
  {
    this.queueJoinTime = queueJoinTime;
  }

  @Override
  public String toString()
  {
    return "Player{" +
            "name='" + name + '\'' +
            ", queueJoinTime=" + queueJoinTime +
            '}';
  }

  public static class PlayerQueueJoinTimeComparator implements Comparator<Player>
  {
    public int compare(Player p1, Player p2)
    {
      if (p1 == p2)
        return 0;
      else if (p1.queueJoinTime < p2.queueJoinTime)
        return -1;
      else if (p1.queueJoinTime > p2.queueJoinTime)
        return 1;
      else if (p1.name != null && p2.name != null)
        return p1.name.compareTo(p2.name);
      else if (p1.name == null)
        return -1;
      else if (p2.name != null)
        return 1;
      else
        return 0;
    }
  }
}
