package net.davidc.egp.snaxe.events;

import net.davidc.egp.common.events.DurationEvent;
import net.davidc.egp.common.interpolators.LinearInterpolatorFloat;
import net.davidc.egp.common.interpolators.LinearInterpolatorInteger;
import net.davidc.egp.snaxe.states.PlayState;
import org.newdawn.slick.AngelCodeFont;
import org.newdawn.slick.Color;
import org.newdawn.slick.Graphics;

/**
 * An event that draws power-up text rising and fading about the given coordinates.
 *
 * @author David Croft (david@davidc.net)
 * @version $Id: PowerUpRiseEvent.java 25 2011-09-18 15:41:54Z david $
 */
public class PowerUpRiseEvent extends DurationEvent<Long, PlayState>
{
  private int x;
  private String text;
  private AngelCodeFont font;

  private LinearInterpolatorInteger yPositionInterpolator;
  private LinearInterpolatorFloat alphaInterpolator;

  public PowerUpRiseEvent(long startTime, long endTime, int x, int y, String text, AngelCodeFont font)
  {
    super(startTime, endTime);
    this.x = x;
    this.text = text;
    this.font = font;
    yPositionInterpolator = new LinearInterpolatorInteger(startTime, endTime, y, y - 20);
    alphaInterpolator = new LinearInterpolatorFloat(startTime + ((endTime - startTime) / 2), endTime, 1, 0);
  }

  @Override
  public void render(Long time, PlayState handle)
  {
    if (!yPositionInterpolator.isValid(time)) return;

    int y = yPositionInterpolator.getValueAtTime(time);

    Graphics g = handle.getGraphics();
    g.setFont(font);
    g.setColor(new Color(0, 0, 0, alphaInterpolator.getValueAtTime(time)));
    g.drawString(text, x, y);
  }
}
