package net.davidc.egp.snaxe.networking;

import java.io.*;
import java.net.Socket;
import java.util.LinkedList;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;

/**
 * @author David Croft (david@davidc.net)
 * @version $Id: UndecimalScreenClient.java 12 2011-09-15 21:51:10Z david $
 */
public class UndecimalScreenClient
{
  private static final Logger log = Logger.getLogger(UndecimalScreenClient.class.getName());

  private static final String NETWORK_CHARSET = "UTF-8";

  private static final String INCOMING_MESSAGE_PLAYER = "m";
  private static final String INCOMING_MESSAGE_NEW = "new";
  private static final String INCOMING_MESSAGE_QUIT = "quit";

  private Socket socket;

  private List<NetworkListener> networkListeners;

  private BufferedReader inputReader;
  private Writer outputWriter;
  private InputStream inputStream;

  public UndecimalScreenClient()
  {
    networkListeners = new LinkedList<NetworkListener>();
  }

  public void init(String host, int port) throws IOException
  {
    socket = new Socket(host, port);
//    socket = SocketChannel.open();
//    socket.configureBlocking(false);

//    socket.connect(new InetSocketAddress(host, port));

//    socket.

    inputStream = socket.getInputStream();
//    inputStream = new SocketChannelInputStream(socket);
    inputReader = new BufferedReader(new InputStreamReader(inputStream, NETWORK_CHARSET));
    OutputStream outputStream = socket.getOutputStream();

    outputWriter = new BufferedWriter(new OutputStreamWriter(outputStream, NETWORK_CHARSET));
    log.log(Level.INFO, "Connected to swarm server " + host + " on port " + port);
  }

  public void addMessageListener(NetworkListener listener)
  {
    networkListeners.add(listener);
  }

  public void removeMessageListener(NetworkListener listener)
  {
    networkListeners.remove(listener);
  }

  // p_playernum_message , or playernum a = all

  public void sendMessage(int playerId, String message) throws IOException
  {
    StringBuilder sb = new StringBuilder();
    sb.append("p_");
    sb.append(playerId);
    sb.append('_');
    sb.append(message);
    sb.append("\r\n");

    log.log(Level.FINEST, "Sending: " + sb.toString());

    outputWriter.write(sb.toString());
    outputWriter.flush();
  }

  public void broadcastMessage(String message) throws IOException
  {
    StringBuilder sb = new StringBuilder();
    sb.append("p_a_");
    sb.append(message);
    sb.append("\r\n");
    
    log.log(Level.FINEST, "Sending: " + sb.toString());

    outputWriter.write(sb.toString());
    outputWriter.flush();
  }

  public void close() throws IOException
  {
    socket.close();
  }

  public void poll() throws IOException
  {
    if (socket.isClosed()) {
      throw new IOException("Server socket is closed");
    }
    if (!socket.isConnected()) {
      throw new IOException("Server socket is not connected");
    }

    // Incoming messages delimited by _, format messagetype_params (e.g. playernumber_message)
    inputStream.available();

    while (inputReader.ready()) {
      String message = inputReader.readLine();

      if (message == null) {
        throw new IOException("END OF STREAM"); // TODO NOT WORKING
      }

      String[] bits = message.split("_", 3);

      if (bits[0].equals(INCOMING_MESSAGE_PLAYER)) {
        // message from player

        int playerId = Integer.parseInt(bits[1]);
        String content = bits[2];

        for (NetworkListener networkListener : networkListeners) {
          networkListener.messageReceived(playerId, content);
        }
      }
      else if (bits[0].equals(INCOMING_MESSAGE_NEW)) {
        int playerId = Integer.parseInt(bits[1]);
        for (NetworkListener networkListener : networkListeners) {
          networkListener.playerJoined(playerId);
        }
      }
      else if (bits[0].equals(INCOMING_MESSAGE_QUIT)) {
        int playerId = Integer.parseInt(bits[1]);
        for (NetworkListener networkListener : networkListeners) {
          networkListener.playerQuit(playerId);
        }
      }
      else {
        throw new RuntimeException("Strange message from server: " + message);
      }
    }
  }
}
